/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "MutationEvent.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomEvent.h"
#include "gdom/GdomMutationEvent.h"
#include "gdom/GdomNode.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomEvent.h"
#include "gdom/PrivateGdomMutationEvent.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateIntGdomMutationEvent.h"
#include "gdom/PrivateIntGdomNode.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMMutationEvent::GDOMMutationEvent(PassRefPtr<WebCore::MutationEvent> impl)
    : GDOMEvent(impl)
{
}

void
gdom_mutation_event_init_mutation_event (GdomMutationEvent *thiz, gchar * type, gboolean can_bubble, gboolean cancelable, GdomNode * related_node, gchar * prev_value, gchar * new_value, gchar * attr_name, gushort attr_change)
{
    g_return_if_fail (thiz);
    WebCore::MutationEvent * item = (gdom_mutation_event_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (type);
    g_return_if_fail (related_node);
    g_return_if_fail (prev_value);
    g_return_if_fail (new_value);
    g_return_if_fail (attr_name);
    WebCore::String _g_type = WebCore::String::fromUTF8(type);
    WebCore::Node * _g_related_node = (gdom_node_get_private(related_node));
    g_return_if_fail (_g_related_node);
    WebCore::String _g_prev_value = WebCore::String::fromUTF8(prev_value);
    WebCore::String _g_new_value = WebCore::String::fromUTF8(new_value);
    WebCore::String _g_attr_name = WebCore::String::fromUTF8(attr_name);
    item->initMutationEvent(_g_type, can_bubble, cancelable, _g_related_node, _g_prev_value, _g_new_value, _g_attr_name, attr_change);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "MutationEvent.h"
#include <gdom/GdomMutationEvent.h>
#include <gdom/PrivateGdomMutationEvent.h>
#include <gdom/PrivateIntGdomMutationEvent.h>
#include <gdom/GdomMutationEventprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomMutationEvent, gdom_mutation_event, GDOM_TYPE_EVENT);

/*
struct _GdomMutationEventPrivate {
    class WebCore::MutationEvent *item;
};

#define GDOM_MUTATION_EVENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_MUTATION_EVENT, GdomMutationEventPrivate))
*/
G_END_DECLS
WebCore::MutationEvent *
gdom_mutation_event_get_private(GdomMutationEvent *request)
{
    GDOMMutationEvent *res;
    res = static_cast<GDOMMutationEvent*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::MutationEvent*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_RELATED_NODE,
    PROP_PREV_VALUE,
    PROP_NEW_VALUE,
    PROP_ATTR_NAME,
    PROP_ATTR_CHANGE,
};


static void gdom_mutation_event_finalize(GObject* object)
{
    /*GdomMutationEvent* request = GDOM_MUTATION_EVENT(object);*/
    /*GdomMutationEventPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_EVENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_mutation_event_parent_class)->finalize(object);
}

static void gdom_mutation_event__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomMutationEvent *self = GDOM_MUTATION_EVENT(object);
    WebCore::MutationEvent * gself = (gdom_mutation_event_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_mutation_event__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomMutationEvent *self = GDOM_MUTATION_EVENT(object);
    WebCore::MutationEvent * gself = (gdom_mutation_event_get_private(self));
    switch (prop_id) {
    case PROP_RELATED_NODE:
    {
         PassRefPtr<WebCore::Node> ptr = gself->relatedNode();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PREV_VALUE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->prevValue()));
         break;
    }
    case PROP_NEW_VALUE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->newValue()));
         break;
    }
    case PROP_ATTR_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->attrName()));
         break;
    }
    case PROP_ATTR_CHANGE:
    {
         g_value_set_uint (value, gself->attrChange());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_mutation_event_class_init(GdomMutationEventClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_mutation_event_finalize;
    g_klass->set_property = gdom_mutation_event__set_property_fn;
    g_klass->get_property = gdom_mutation_event__get_property_fn;

    g_object_class_install_property (g_klass, PROP_RELATED_NODE,
            g_param_spec_object ("related_node", /* name */
                "mutation_event_related_node", /* short description */
                "read-only  GdomNode * MutationEvent.related_node", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PREV_VALUE,
            g_param_spec_string ("prev_value", /* name */
                "mutation_event_prev_value", /* short description */
                "read-only  gchar * MutationEvent.prev_value", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_NEW_VALUE,
            g_param_spec_string ("new_value", /* name */
                "mutation_event_new_value", /* short description */
                "read-only  gchar * MutationEvent.new_value", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ATTR_NAME,
            g_param_spec_string ("attr_name", /* name */
                "mutation_event_attr_name", /* short description */
                "read-only  gchar * MutationEvent.attr_name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ATTR_CHANGE,
            g_param_spec_uint ("attr_change", /* name */
                "mutation_event_attr_change", /* short description */
                "read-only  gushort MutationEvent.attr_change", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXUINT16, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomMutationEventPrivate));*/
}

static void gdom_mutation_event_init(GdomMutationEvent* request)
{
    /*GdomMutationEventPrivate* priv = GDOM_MUTATION_EVENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

